# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

#!/usr/bin/env python

import GameUIController
import sys

class Delegate(object):
    def print_board(self, gstate):
        print
        # get max string size so we can pad to it
        maxlen = max(len(str(val)) for val in gstate.grid.flat)
        
        def pad(v):
            s = str(v)
            return ' '*(maxlen-len(s)) + s
            
        for y in range(gstate.grid.shape[1]-1, -1, -1):
            line = [gstate.grid[x,y] for x in range(gstate.grid.shape[0])]
            print(' '.join(pad(val) for val in line))
        if gstate.has_scores():
            for p in range(gstate.num_players):
                print('%sPlayer %s: %s' % ('*' if gstate.current_player==p+1 else ' ', str(p+1), gstate.player_score(p+1), ))
    
    def made_move(self, controller, prevstate=None, animations=None):
        if controller.is_human_turn() or controller.is_network_turn():
            self.print_board(controller.current_game_state())
        if controller.is_network_turn():
            print('Waiting for network player...')

    def start_ai_move(self, controller):
        controller.do_ai_move()
        
    def made_ai_move(self, controller, prevstate=None, animations=None):
        self.made_move(controller)
        
    def do_game_over(self, controller):
        self.print_board(controller)
        print('Player %s wins' % (controller.current_game_state().winning_player(),))
        
    def network_connected(self, controller, isserver=True):
        if isserver: controller.start_game()
    

# chop s into tuples of size ndim, so "1 2 3 4 5 6" with ndim=2 becomes ((1,2),(3,4),(5,6))
def move_for_string(s, ndim):
        vals = s.split()
        # allow input as single number without spaces (assuming no more than 10 per dimension)
        if len(vals)==1: vals = list(vals[0])
        return tuple(tuple(map(int, vals[i*ndim:(i+1)*ndim])) for i in xrange(len(vals)//ndim))

def main():
    if len(sys.argv)<2 or sys.argv[1]=='--help':
        print('Usage: pygrid_console <Game> [AI]')
        sys.exit(0)
        
    game = sys.argv[1] if len(sys.argv)>1 else None
    ai = sys.argv[2] if len(sys.argv)>2 else None
    
    controller = GameUIController.Controller(Delegate())
    controller.game_config.select_game(game)
    if ai: controller.game_config.set_ai_for_player(ai, 2)
    controller.start_game()
    done = False
    while not done:
        try:
            s = raw_input(controller.game_config.game_name+'> ').strip()
            if not s: continue
            if s in ('quit','exit'): 
                done = True
                continue
            if s.startswith('load '):
                path = s[5:].strip()
                controller.load_game(path)
                controller.made_move()
                continue
            if s.startswith('save '):
                path = s[5:].strip()
                controller.save_game(path)
                continue
            if s.startswith('new '):
                args = s.split()
                newgame = args[1] if len(args)>1 else game
                variation = args[2] if len(args)>2 else None
                controller.game_config.select_game(newgame, variation=variation)
                controller.start_game()
                continue
            if s.startswith('host '):
                port = int(s.split()[1])
                controller.network_listen(port)
                continue
            if s.startswith('join '):
                fields = s.split()
                host = fields[1]
                port = int(fields[2]) if len(fields)>=3 else 27500
                controller.network_connect(host, port)
                continue
            move = move_for_string(s, len(controller.current_game_state().grid.shape))
            controller.make_human_move(move)
            if controller.is_game_over(): 
                done = True
        except:
            import traceback
            traceback.print_exc()
            
if __name__=='__main__':
    main()
                